#include "ui.hxx"

using namespace ge211;

// Converts a logical board position to the physical screen position
// of the upper-left corner of the corresponding grid square.
static Position board_to_screen_(Position board_pos)
{
    int x = 2 * token_radius * board_pos.x;
    int y = 2 * token_radius * (Connect4_model::n - board_pos.y - 1);
    return {x, y};
}

// Converts a physical screen position to the logical board position
// that corresponds to it.
static Position screen_to_board_(Position screen_pos)
{
    int col_no = screen_pos.x / (2 * token_radius);
    int row_no = Connect4_model::n - screen_pos.y / (2 * token_radius) - 1;
    return {col_no, row_no};
}

void Connect4_ui::draw(ge211::Sprite_set& sprites)
{
    // Here we add a sprite for each token in the game:
    for (int col_no = 0; col_no < Connect4_model::m; ++col_no) {
        const Connect4_model::column_t& column = model_.column(col_no);
        for (int row_no = 0; row_no < column.size(); ++row_no) {
            Player player = column[row_no];
            Position screen_pos = board_to_screen_({col_no, row_no});

            if (player == Player::first)
                sprites.add_sprite(player1_token_, screen_pos);
            else
                sprites.add_sprite(player2_token_, screen_pos);
        }
    }

    // Here we possibly add a sprite as a cursor that follows the mouse:
    if (model_.is_playable(mouse_column_)) {
        int      col_no     = mouse_column_,
                 row_no     = (int) model_.column(col_no).size();
        Position screen_pos = board_to_screen_({col_no, row_no});

        if (model_.turn() == Player::first)
            sprites.add_sprite(player1_shadow_, screen_pos);
        else
            sprites.add_sprite(player2_shadow_, screen_pos);
    }

    // Select the background color for the window based on the
    // winner or lack thereof:
    if (model_.winner() == Player::first)
        background_color = first_tint;
    else if (model_.winner() == Player::second)
        background_color = second_tint;
    else if (model_.is_game_over())
        background_color = stalemate_bg;
    else
        background_color = playing_bg;
}

void Connect4_ui::on_mouse_move(Position screen_pos)
{
    mouse_column_ = screen_to_board_(screen_pos).x;
}

Dimensions Connect4_ui::initial_window_dimensions() const
{
    return board_pixels();
}

ge211::Dimensions Connect4_ui::board_pixels()
{
    return {2 * token_radius * Connect4_model::m,
            2 * token_radius * Connect4_model::n};
}

std::string Connect4_ui::initial_window_title() const
{
    return "Connect Four";
}

void Connect4_ui::on_mouse_down(Mouse_button btn, Position screen_posn)
{
    if (model_.turn() == Player::neither || btn != Mouse_button::left)
        return;

    int col_no = screen_to_board_(screen_posn).x;

    if (model_.is_playable(col_no))
        model_.place_token(col_no);
}
